// @TITLE "matmul.cc -- matrix multiply"
//
// matmul -- a parallel matrix-matrix multiply program
//   written using the Data-Parallel Programming Library for Education (DAPPLE)
//
// This version is fairly straightforward, but it has limited parallelism.
// For each entry in the result matrix, we do a parallel dot product
// of the appropriate row and the appropriate column.
// Thus, it takes N^2 iterations each with N-way parallelism, 
// for an NxN matrix.
//
// David Kotz 1994

// $Id: matmul.cc,v 1.7 95/12/17 00:08:15 dfk release Locker: dfk $

#include <stdlib.h>
#include <iostream.h>
#include <math.h>
#include "dapple.h"

// we'll multiply a PxQ matrix by a QxR matrix to get a PxR matrix
const int P = 3;
const int Q = 3;
const int R = 3;

int
main()
{
    // we'll compute C = A * B
    intMatrix A(P,Q);
    intMatrix B(Q,R);
    intMatrix C(P,R);
    intMatrix D(P,R);		// D is what C should be
    
    // load matrices; row-major order, whitespace-separated integers
    cout << "To multiply C = A * B." << endl;
    cout << "Enter " << P << " by " << Q << " matrix A: " << endl;
    cin >> A;
    cout << "Enter " << Q << " by " << R << " matrix B: " << endl;
    cin >> B;

    // loop through the result locations
    for (int r = 0; r < P; r++)
      for (int c = 0; c < R; c++)
	C[r][c] = inner(A[r][_], B[_][c]);

    cout << "Result C = " << endl;
    cout << C;

    cout << "Enter correct " << P << " by " << R << " matrix D: " << endl;
    cin >> D;

    if (any(C != D))
      cout << "The answers are different." << endl;
    else
      cout << "The answers are the same!" << endl;

    return(0);
}
