// @TITLE "BigSmall.h: multi-type Biggest and Smallest"
//
// BigSmall - some simple constants that represent the largest and smallest
// values for many types.
// It is used by Collection and Vector and others, for min and max operations.
//
// David Kotz 1994

// $Id $

#ifndef BIGSMALL_H
#define BIGSMALL_H

#include <math.h>
#include "macros.h"

#ifdef __MWERKS__
#include <limits.h>
#include <float.h>
#else
# ifndef FLT_MAX
#  define FLT_MAX MAXFLOAT
# endif
# ifndef FLT_MIN
#  define FLT_MIN -MAXFLOAT
# endif
#endif

// should work anywhere that uses two's complement arithmetic.
#define MAXINT ((int)( (((unsigned int) (-1)) << 1) >> 1 ))

// To get the biggest value of type T, use: T big = Biggest();

class Biggest {
public:
    operator int() {return (int) MAXINT; }
    operator char() {return (char) 0127; }
    operator float() {return (float) FLT_MAX; }
    operator double() {return (double) HUGE_VAL; }
    operator boolean() {return true; }
};

// To get the smallest value of type T, use: T small = Smallest();

class Smallest {
public:
    operator int() {return (int) -MAXINT; }
    operator char() {return (char) 0; }
    operator float() {return (float) FLT_MIN; }
    operator double() {return (double) -HUGE_VAL; }
    operator boolean() {return false; }
};

#endif // BIGSMALL_H
